@@ -22,8 +22,6 @@ static pgssSharedState *pgss;
 static HTAB *pgss_hash;
 static HTAB *pgss_query_hash;
 
-
-
 static HTAB* hash_init(const char *hash_name, int key_size, int entry_size, int hash_size);
 
 static HTAB*
@@ -72,7 +70,7 @@ pgss_startup(void)
 	}
 
 	pgss_hash = hash_init("pg_stat_monitor: bucket hashtable", sizeof(pgssHashKey), sizeof(pgssEntry), MAX_BUCKET_ENTRIES);
-	pgss_query_hash = hash_init("pg_stat_monitor: query hashtable", sizeof(pgssQueryHashKey), sizeof(pgssQueryEntry),500000);
+	pgss_query_hash = hash_init("pg_stat_monitor: query hashtable", sizeof(pgssQueryHashKey), sizeof(pgssQueryEntry),MAX_BUCKET_ENTRIES);
 
 	LWLockRelease(AddinShmemInitLock);
 
@@ -95,6 +93,12 @@ pgsm_get_hash(void)
 	return pgss_hash;
 }
 
+HTAB*
+pgsm_get_query_hash(void)
+{
+	return pgss_query_hash;
+}
+
 /*
  * shmem_shutdown hook: Dump statistics into file.
  *
@@ -122,12 +126,11 @@ hash_memsize(void)
 	size = MAXALIGN(sizeof(pgssSharedState));
 	size += MAXALIGN(MAX_QUERY_BUF);
 	size = add_size(size, hash_estimate_size(MAX_BUCKET_ENTRIES, sizeof(pgssEntry)));
-	size = add_size(size, hash_estimate_size(500000, sizeof(pgssQueryEntry)));
+	size = add_size(size, hash_estimate_size(MAX_BUCKET_ENTRIES, sizeof(pgssQueryEntry)));
 
 	return size;
 }
 
-
 pgssEntry *
 hash_entry_alloc(pgssSharedState *pgss, pgssHashKey *key,int encoding)
 {
@@ -135,8 +138,10 @@ hash_entry_alloc(pgssSharedState *pgss, pgssHashKey *key,int encoding)
 	bool		found = false;
 
 	if (hash_get_num_entries(pgss_hash) >= MAX_BUCKET_ENTRIES)
+	{
+		elog(DEBUG1, "%s", "pg_stat_monitor: out of memory");
 		return NULL;
-
+	}
 	/* Find or create an entry with desired hash code */
 	entry = (pgssEntry *) hash_search(pgss_hash, key, HASH_ENTER, &found);
 	if (!found)
@@ -152,7 +157,7 @@ hash_entry_alloc(pgssSharedState *pgss, pgssHashKey *key,int encoding)
 		entry->encoding = encoding;
 	}
 	if (entry == NULL)
-		elog(FATAL, "%s", "pg_stat_monitor: out of memory");
+		elog(DEBUG1, "%s", "pg_stat_monitor: out of memory");
 	return entry;
 }
 
@@ -170,7 +175,7 @@ hash_query_entry_dealloc(int bucket)
 	hash_seq_init(&hash_seq, pgss_query_hash);
 	while ((entry = hash_seq_search(&hash_seq)) != NULL)
 	{
-		if (entry->key.bucket_id == bucket)
+		if (entry->key.bucket_id == bucket || bucket < 0)
 			entry = hash_search(pgss_query_hash, &entry->key, HASH_REMOVE, NULL);
 	}
 }
@@ -218,8 +223,8 @@ hash_entry_reset()
 }
 
 /* Caller must accuire lock */
-bool
-hash_create_query_entry(uint64 bucket_id, uint64 queryid)
+pgssQueryEntry*
+hash_create_query_entry(uint64 bucket_id, uint64 queryid, uint64 dbid, uint64 userid, uint64 ip)
 {
     pgssQueryHashKey    key;
 	pgssQueryEntry      *entry;
@@ -227,14 +232,17 @@ hash_create_query_entry(uint64 bucket_id, uint64 queryid)
 
     key.queryid = queryid;
 	key.bucket_id = bucket_id;
+	key.dbid = dbid;
+	key.userid = userid;
+	key.ip = ip;
 
 	entry = (pgssQueryEntry *) hash_search(pgss_query_hash, &key, HASH_ENTER, &found);
-	return (entry != NULL);
+	return entry;
 }
 
 /* Caller must accuire lock */
-bool
-hash_find_query_entry(uint64 bucket_id, uint64 queryid)
+pgssQueryEntry*
+hash_find_query_entry(uint64 bucket_id, uint64 queryid, uint64 dbid, uint64 userid, uint64 ip)
 {
     pgssQueryHashKey    key;
 	pgssQueryEntry      *entry;
@@ -242,10 +250,13 @@ hash_find_query_entry(uint64 bucket_id, uint64 queryid)
 
     key.queryid = queryid;
 	key.bucket_id = bucket_id;
+	key.dbid = dbid;
+	key.userid = userid;
+	key.ip = ip;
 
     /* Lookup the hash table entry with shared lock. */
 	entry = (pgssQueryEntry *) hash_search(pgss_query_hash, &key, HASH_FIND, &found);
-    return ((entry != NULL) && found);
+    return entry;
 }
 
 bool
