@@ -24,40 +24,43 @@ RETURNS text[] AS $$
 SELECT string_to_array(get_histogram_timings(), ','); 
 $$ LANGUAGE SQL;
 
-CREATE FUNCTION pg_stat_monitor(IN showtext boolean,
-    OUT bucket              int,
+CREATE FUNCTION pg_stat_monitor_internal(IN showtext boolean,
+    OUT bucket              int8,   -- 0
     OUT userid              oid,
     OUT dbid                oid,
     OUT client_ip           int8,
 
-    OUT queryid             text,
-    OUT parentid            text,
+    OUT queryid             text,  -- 4
+    OUT planid              text,
     OUT query               text,
+    OUT query_plan          text,
+    OUT state_code 			int8,
+    OUT top_queryid         text,
 	OUT application_name	text,
-	OUT relations			text,
+
+	OUT relations			text, -- 11
 	OUT cmd_type			int,
 	OUT elevel              int,
     OUT sqlcode             TEXT,
     OUT message             text,
     OUT bucket_start_time   text,
 
-	OUT plans          		int8,
-    OUT plan_total_time     float8,
-    OUT plan_min_time       float8,
-    OUT plan_max_time       float8,
-    OUT plan_mean_time      float8,
-    OUT plan_stddev_time    float8,
-    OUT plan_rows           int8,
-
-	OUT calls         		int8,
+	OUT calls         		int8,  -- 16
     OUT total_time          float8,
     OUT min_time            float8,
     OUT max_time            float8,
     OUT mean_time           float8,
     OUT stddev_time         float8,
-    OUT rows       			int8,
+    OUT rows_retrieved       int8,
 
-	OUT shared_blks_hit     int8,
+	OUT plans_calls    	 	int8,  -- 23
+    OUT plan_total_time     float8,
+    OUT plan_min_time       float8,
+    OUT plan_max_time       float8,
+    OUT plan_mean_time      float8,
+    OUT plan_stddev_time    float8,
+
+	OUT shared_blks_hit     int8, -- 29
     OUT shared_blks_read    int8,
     OUT shared_blks_dirtied int8,
     OUT shared_blks_written int8,
@@ -69,7 +72,7 @@ CREATE FUNCTION pg_stat_monitor(IN showtext boolean,
     OUT temp_blks_written   int8,
     OUT blk_read_time       float8,
     OUT blk_write_time      float8,
-    OUT resp_calls          text,
+    OUT resp_calls          text, -- 41
     OUT cpu_user_time       float8,
     OUT cpu_sys_time        float8,
     OUT wal_records 		int8,
@@ -80,6 +83,19 @@ RETURNS SETOF record
 AS 'MODULE_PATHNAME', 'pg_stat_monitor'
 LANGUAGE C STRICT VOLATILE PARALLEL SAFE;
 
+CREATE OR REPLACE FUNCTION get_state(state_code int8) RETURNS TEXT AS
+$$
+SELECT
+	CASE
+		WHEN state_code = 0 THEN 'PARSING'
+		WHEN state_code = 1 THEN 'PLANNING'
+		WHEN state_code = 2 THEN 'ACTIVE'
+		WHEN state_code = 3 THEN 'FINISHED'
+		WHEN state_code = 4 THEN 'FINISHED WITH ERROR'
+	END
+$$
+LANGUAGE SQL PARALLEL SAFE;
+
 CREATE or REPLACE FUNCTION get_cmd_type (cmd_type INTEGER) RETURNS TEXT AS
 $$
 SELECT
@@ -126,9 +142,11 @@ CREATE VIEW pg_stat_monitor AS SELECT
     datname,
 	'0.0.0.0'::inet + client_ip AS client_ip,
     queryid,
-    parentid,
+    top_queryid,
     query,
-	(SELECT query from pg_stat_monitor(true) s where s.queryid = p.parentid) AS parent_query,
+	planid,
+	query_plan,
+	(SELECT query from pg_stat_monitor_internal(true) s where s.queryid = p.top_queryid) AS top_query,
 	application_name,
 	string_to_array(relations, ',') AS relations,
 	cmd_type,
@@ -136,20 +154,20 @@ CREATE VIEW pg_stat_monitor AS SELECT
 	elevel,
 	sqlcode,
 	message,
-	plans,
-	round( CAST(plan_total_time as numeric), 4)::float8 as plan_total_time,
-	round( CAST(plan_min_time as numeric), 4)::float8 as plan_min_time,
-	round( CAST(plan_max_time as numeric), 4)::float8 as plan_max_time,
-	round( CAST(plan_mean_time as numeric), 4)::float8 as plan_mean_time,
-	round( CAST(plan_stddev_time as numeric), 4)::float8 as plan_stddev_time,
     calls,
 	round( CAST(total_time as numeric), 4)::float8 as total_time,
 	round( CAST(min_time as numeric), 4)::float8 as min_time,
 	round( CAST(max_time as numeric), 4)::float8 as max_time,
 	round( CAST(mean_time as numeric), 4)::float8 as mean_time,
 	round( CAST(stddev_time as numeric), 4)::float8 as stddev_time,
-	rows,
-    shared_blks_hit,
+	rows_retrieved,
+	plans_calls,
+	round( CAST(plan_total_time as numeric), 4)::float8 as plan_total_time,
+	round( CAST(plan_min_time as numeric), 4)::float8 as plan_min_time,
+	round( CAST(plan_max_time as numeric), 4)::float8 as plan_max_time,
+	round( CAST(plan_mean_time as numeric), 4)::float8 as plan_mean_time,
+ 
+	shared_blks_hit,
     shared_blks_read,
     shared_blks_dirtied,
     shared_blks_written,
@@ -166,8 +184,10 @@ CREATE VIEW pg_stat_monitor AS SELECT
 	round(cpu_sys_time::numeric, 4) as cpu_sys_time,
     wal_records,
     wal_fpi,
-    wal_bytes
-FROM pg_stat_monitor(TRUE) p, pg_database d  WHERE dbid = oid
+    wal_bytes,
+	state_code,
+	get_state(state_code) as state
+FROM pg_stat_monitor_internal(TRUE) p, pg_database d  WHERE dbid = oid
 ORDER BY bucket_start_time;
 
 CREATE FUNCTION decode_error_level(elevel int)
