@@ -79,19 +79,26 @@
 #define CMD_LEN				20
 #define APPLICATIONNAME_LEN	100
 #define PGSM_OVER_FLOW_MAX	10
-
+#define PLAN_TEXT_LEN		1024
 /* the assumption of query max nested level */
 #define DEFAULT_MAX_NESTED_LEVEL	10
 
 #define MAX_QUERY_BUF						(PGSM_QUERY_SHARED_BUFFER * 1024 * 1024)
 #define MAX_BUCKETS_MEM 					(PGSM_MAX * 1024 * 1024)
 #define BUCKETS_MEM_OVERFLOW() 				((hash_get_num_entries(pgss_hash) * sizeof(pgssEntry)) >= MAX_BUCKETS_MEM)
-#define MAX_QUERY_BUFFER_BUCKET			MAX_QUERY_BUF / PGSM_MAX_BUCKETS
+#define MAX_QUERY_BUFFER_BUCKET			    MAX_QUERY_BUF / PGSM_MAX_BUCKETS
 #define MAX_BUCKET_ENTRIES 					(MAX_BUCKETS_MEM / sizeof(pgssEntry))
 #define QUERY_BUFFER_OVERFLOW(x,y)  		((x + y + sizeof(uint64) + sizeof(uint64)) > MAX_QUERY_BUFFER_BUCKET)
 #define QUERY_MARGIN 						100
 #define MIN_QUERY_LEN						10
 #define SQLCODE_LEN                         20
+
+#if PG_VERSION_NUM >= 130000
+#define	MAX_SETTINGS                        14
+#else
+#define MAX_SETTINGS                        13
+#endif
+
 typedef struct GucVariables
 {
 	int		guc_variable;
@@ -105,6 +112,21 @@ typedef struct GucVariables
 	bool 	guc_restart;
 } GucVariable;
 
+#if PG_VERSION_NUM < 130000
+typedef struct WalUsage
+{
+    long        wal_records;    /* # of WAL records produced */
+    long        wal_fpi;        /* # of WAL full page images produced */
+    uint64      wal_bytes;      /* size of WAL records produced */
+} WalUsage;
+#endif
+
+typedef enum OVERFLOW_TARGET
+{
+	OVERFLOW_TARGET_NONE = 0,
+	OVERFLOW_TARGET_DISK
+} OVERFLOW_TARGET;
+
 typedef enum pgssStoreKind
 {
 	PGSS_INVALID = -1,
@@ -114,8 +136,11 @@ typedef enum pgssStoreKind
 	 * reference the underlying values in the arrays in the Counters struct,
 	 * and this order is required in pg_stat_statements_internal().
 	 */
-	PGSS_PLAN = 0,
+	PGSS_PARSE = 0,
+	PGSS_PLAN,
 	PGSS_EXEC,
+	PGSS_FINISHED,
+	PGSS_ERROR,
 
 	PGSS_NUMKIND				/* Must be last value of this enum */
 } pgssStoreKind;
@@ -136,18 +161,36 @@ typedef enum AGG_KEY
 #define MAX_QUERY_LEN 1024
 
 /* shared nenory storage for the query */
+typedef struct CallTime
+{
+	double		total_time;					/* total execution time, in msec */
+	double		min_time;					/* minimum execution time in msec */
+	double		max_time;					/* maximum execution time in msec */
+	double		mean_time;					/* mean execution time in msec */
+	double		sum_var_time;				/* sum of variances in execution time in msec */
+} CallTime;
+
 typedef struct pgssQueryHashKey
 {
-	uint64		queryid;		/* query identifier */
 	uint64		bucket_id;		/* bucket number */
+	uint64		queryid;		/* query identifier */
+	uint64		userid;			/* user OID */
+	uint64		dbid;			/* database OID */
+	uint64		ip;				/* client ip address */
 } pgssQueryHashKey;
 
 typedef struct pgssQueryEntry
 {
-	pgssQueryHashKey	key;		/* hash key of entry - MUST BE FIRST */
-	uint64			    pos;		/* bucket number */
+	pgssQueryHashKey    key;		/* hash key of entry - MUST BE FIRST */
+	uint64              pos;		/* bucket number */
 } pgssQueryEntry;
 
+typedef struct PlanInfo
+{
+	uint64		planid;						/* plan identifier */
+	char 		plan_text[PLAN_TEXT_LEN];	/* plan text */
+} PlanInfo;
+
 typedef struct pgssHashKey
 {
 	uint64		bucket_id;		/* bucket number */
@@ -155,6 +198,7 @@ typedef struct pgssHashKey
 	uint64		userid;			/* user OID */
 	uint64		dbid;			/* database OID */
 	uint64		ip;				/* client ip address */
+	uint64		planid;			/* plan identifier */
 } pgssHashKey;
 
 typedef struct QueryInfo
@@ -174,7 +218,7 @@ typedef struct QueryInfo
 typedef struct ErrorInfo
 {
 	int64	elevel;							/* error elevel */
-	char    sqlcode[SQLCODE_LEN];						/* error sqlcode  */
+	char    sqlcode[SQLCODE_LEN];			/* error sqlcode  */
 	char	message[ERROR_MESSAGE_LEN];   	/* error message text */
 } ErrorInfo;
 
@@ -185,14 +229,6 @@ typedef struct Calls
 	double		usage;						/* usage factor */
 } Calls;
 
-typedef struct CallTime
-{
-	double		total_time;					/* total execution time, in msec */
-	double		min_time;					/* minimum execution time in msec */
-	double		max_time;					/* maximum execution time in msec */
-	double		mean_time;					/* mean execution time in msec */
-	double		sum_var_time;				/* sum of variances in execution time in msec */
-} CallTime;
 
 typedef struct Blocks
 {
@@ -222,21 +258,24 @@ typedef struct Wal_Usage
 	int64		wal_fpi;		/* # of WAL full page images generated */
 	uint64		wal_bytes;		/* total amount of WAL bytes generated */
 } Wal_Usage;
-/*
- * The actual stats counters kept within pgssEntry.
- */
+
 typedef struct Counters
 {
 	uint64		bucket_id;		/* bucket id */
-	Calls		calls[PGSS_NUMKIND];
+	Calls		calls;
 	QueryInfo	info;
-	CallTime	time[PGSS_NUMKIND];
+	CallTime	time;
+
+	Calls		plancalls;
+	CallTime	plantime;
+	PlanInfo    planinfo;
+
 	Blocks		blocks;
 	SysInfo		sysinfo;
 	ErrorInfo   error;
-	Wal_Usage    walusage;
-	int			plans;
+	Wal_Usage   walusage;
 	int			resp_calls[MAX_RESPONSE_BUCKET];	/* execution time's in msec */
+	uint64		state;		/* query state */
 } Counters;
 
 /* Some global structure to get the cpu usage, really don't like the idea of global variable */
@@ -327,7 +366,10 @@ void pgss_shmem_startup(void);
 void pgss_shmem_shutdown(int code, Datum arg);
 int pgsm_get_bucket_size(void);
 pgssSharedState* pgsm_get_ss(void);
-HTAB* pgsm_get_hash(void);
+HTAB *pgsm_get_plan_hash(void);
+HTAB *pgsm_get_query_hash(void);
+HTAB *pgsm_get_hash(void);
+HTAB *pgsm_get_plan_hash(void);
 HTAB* pgsm_get_query_hash(void);
 void hash_entry_reset(void);
 void hash_query_entry_dealloc(int bucket);
@@ -335,10 +377,8 @@ void hash_entry_dealloc(int bucket);
 pgssEntry* hash_entry_alloc(pgssSharedState *pgss, pgssHashKey *key, int encoding);
 Size hash_memsize(void);
 
-bool hash_find_query_entry(uint64 bucket_id, uint64 queryid);
-bool hash_create_query_entry(uint64 bucket_id, uint64 queryid);
-
-/* hash_query.c */
+pgssQueryEntry* hash_find_query_entry(uint64 bucket_id, uint64 queryid, uint64 dbid, uint64 userid, uint64 ip);
+pgssQueryEntry* hash_create_query_entry(uint64 bucket_id, uint64 queryid, uint64 dbid, uint64 userid, uint64 ip);
 void pgss_startup(void);
 void set_qbuf(int i, unsigned char *);
 
@@ -356,6 +396,8 @@ void pgss_startup(void);
 #define PGSM_HISTOGRAM_MAX get_conf(8)->guc_variable
 #define PGSM_HISTOGRAM_BUCKETS get_conf(9)->guc_variable
 #define PGSM_QUERY_SHARED_BUFFER get_conf(10)->guc_variable
-#define PGSM_TRACK_PLANNING get_conf(11)->guc_variable
+#define PGSM_OVERFLOW_TARGET get_conf(12)->guc_variable
+#define PGSM_QUERY_PLAN get_conf(12)->guc_variable
+#define PGSM_TRACK_PLANNING get_conf(13)->guc_variable
 
 #endif
